@@ -59,8 +59,8 @@ METHOD(public_key_t, get_type, key_type_t,
 /**
  * Verify a BLISS signature based on a SHA-512 hash
  */
-static bool verify_bliss_with_sha512(private_bliss_public_key_t *this,
-									 chunk_t data, chunk_t signature)
+static bool verify_bliss(private_bliss_public_key_t *this, hash_algorithm_t alg,
+						 chunk_t data, chunk_t signature)
 {
 	int i, n;
 	int32_t *z1, *u;
@@ -68,39 +68,44 @@ static bool verify_bliss_with_sha512(private_bliss_public_key_t *this,
 	uint16_t q, q2, p, *c_indices, *indices;
 	uint32_t *az;
 	uint8_t data_hash_buf[HASH_SIZE_SHA512];
-	chunk_t data_hash = { data_hash_buf, sizeof(data_hash_buf) };
+	chunk_t data_hash;
 	hasher_t *hasher;
+	hash_algorithm_t oracle_alg;
 	bliss_fft_t *fft;
 	bliss_signature_t *sig;
 	bool success = FALSE;
 
-	/* Create data hash */
-	hasher = lib->crypto->create_hasher(lib->crypto, HASH_SHA512);
+	/* Create data hash using configurable hash algorithm */
+	hasher = lib->crypto->create_hasher(lib->crypto, alg);
 	if (!hasher )
 	{
 		return FALSE;
 	}
+	data_hash = chunk_create(data_hash_buf, hasher->get_hash_size(hasher));
+
 	if (!hasher->get_hash(hasher, data, data_hash_buf))
 	{
 		hasher->destroy(hasher);
 		return FALSE;
 	}
+	hasher->destroy(hasher);
 
 	sig = bliss_signature_create_from_data(this->set, signature);
 	if (!sig)
 	{
-		hasher->destroy(hasher);
 		return FALSE;
 	}
 	sig->get_parameters(sig, &z1, &z2d, &c_indices);
 
 	if (!bliss_utils_check_norms(this->set, z1, z2d))
 	{
-		hasher->destroy(hasher);
 		sig->destroy(sig);
 		return FALSE;
 	}
 
+	/* MGF1 hash algorithm to be used for random oracle */
+	oracle_alg = HASH_SHA512;
+
 	/* Initialize a couple of needed variables */
 	n  = this->set->n;
 	q  = this->set->q;
@@ -155,8 +160,7 @@ static bool verify_bliss_with_sha512(private_bliss_public_key_t *this,
 		DBG3(DBG_LIB, "%3d  %6d   %4d  %4d", i, u[i], ud[i], z2d[i]);
 	}
 
-	if (!bliss_utils_generate_c(hasher, data_hash, ud, n, this->set->kappa,
-								indices))
+	if (!bliss_utils_generate_c(oracle_alg, data_hash, ud, this->set, indices))
 	{
 		goto end;
 	}
@@ -173,7 +177,6 @@ static bool verify_bliss_with_sha512(private_bliss_public_key_t *this,
 
 end:
 	/* cleanup */
-	hasher->destroy(hasher);
 	sig->destroy(sig);
 	fft->destroy(fft);
 	free(az);
@@ -190,8 +193,12 @@ METHOD(public_key_t, verify, bool,
 {
 	switch (scheme)
 	{
+		case SIGN_BLISS_WITH_SHA256:
+			return verify_bliss(this, HASH_SHA256, data, signature);
+		case SIGN_BLISS_WITH_SHA384:
+			return verify_bliss(this, HASH_SHA384, data, signature);
 		case SIGN_BLISS_WITH_SHA512:
-			return verify_bliss_with_sha512(this, data, signature);
+			return verify_bliss(this, HASH_SHA512, data, signature);
 		default:
 			DBG1(DBG_LIB, "signature scheme %N not supported by BLISS",
 				 signature_scheme_names, scheme);
@@ -410,6 +417,11 @@ bool bliss_public_key_from_asn1(chunk_t object, bliss_param_set_t *set,
 	for (i = 0; i < set->n; i++)
 	{
 		packer->read_bits(packer, &coefficient, set->q_bits);
+		if (coefficient >= set->q)
+		{
+			packer->destroy(packer);
+			return FALSE;
+		}
 		(*pubkey)[i] = coefficient;
 	}
 	packer->destroy(packer);
